# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re

from scss.compiler import compile_file  # type: ignore
import flask

from aep_site.jinja.env import jinja_env
from aep_site.models.site import Site


ROOT = os.path.realpath(os.path.join(os.path.dirname(__file__), ".."))
app = flask.Flask("aep", static_folder=f"{ROOT}/aep_site/support/assets/")
app.jinja_env = jinja_env  # type: ignore


@app.route("/")
def hello():
    return flask.render_template("index.html.j2", site=flask.g.site)


@app.route("/<int:aep_id>")
def aep(aep_id: int):
    """Display a single AEP document."""
    return flask.g.site.aeps[aep_id].render()


@app.route("/search")
def search():
    """Display the search page."""
    return flask.render_template(
        "search.html.j2",
        site=flask.g.site,
        path=flask.request.path,
    )


@app.route("/<page>")
@app.route("/<collection>/<page>")
def page(page: str, collection: str = "general"):
    """Display a static page or listing of AEPs in a given scope."""
    site = flask.g.site
    if page in site.scopes:
        return site.scopes[page].render()
    if page in site.aeps_by_slug:
        return site.aeps_by_slug[page].render()
    if collection in site.collections and page in site.collections[collection].pages:
        return site.collections[collection].pages[page].render()
    if collection in site.scopes and int(page) in site.scopes[collection].aeps:
        return site.scopes[collection].aeps[int(page)].render()
    flask.abort(404)


@app.route("/assets/css/<path:css_file>")
def scss(css_file: str):
    """Compile the given SCSS file and return it."""
    scss_file = re.sub(r"\.css$", ".scss", css_file)
    css = compile_file(f"{ROOT}/aep_site/support/scss/{scss_file}")
    return flask.Response(css, mimetype="text/css")


@app.route("/assets/js/search/tipuesearch_content.js")
def search_content():
    """Compile the search content JavaScript and return it."""
    return flask.Response(
        flask.render_template("search.js.j2", site=flask.g.site),
        mimetype="text/javascript",
    )


def site_load_func(src: str):
    """Return a function that loads the site and registers it.

    This is used in cli.py to register to Flask.before_request.
    """

    def fx():
        flask.g.site = Site.load(src)

        # This is the dev server, so plow over whatever the configuration
        # says that the site URL is.
        flask.g.site.config.setdefault("urls", {})
        flask.g.site.config["urls"]["site"] = "http://localhost:4000"

    return fx



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import click

from aep_site import server
from aep_site.publisher import Publisher


@click.command()
@click.argument(
    "src",
    type=click.Path(
        allow_dash=False,
        dir_okay=True,
        exists=True,
        file_okay=False,
    ),
)
@click.argument(
    "dest",
    type=click.Path(
        allow_dash=False, dir_okay=True, exists=False, file_okay=False, writable=True
    ),
)
def publish(src: str, dest: str):
    """Publish static HTML pages into the specified location."""
    publisher = Publisher(src, dest)
    publisher.publish_site()


@click.command()
@click.argument(
    "src",
    type=click.Path(allow_dash=False, dir_okay=True, exists=True, file_okay=False),
)
def serve(src: str):
    """Run a debug server."""
    server.app.before_request(server.site_load_func(src))
    server.app.run(host="0.0.0.0", port=4000, debug=True)



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import io
import os

from setuptools import find_packages, setup  # type: ignore


PACKAGE_ROOT = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(PACKAGE_ROOT, "VERSION"), "r") as version_file:
    VERSION = version_file.read().strip()

with io.open(os.path.join(PACKAGE_ROOT, "README.md"), "r") as readme_file:
    long_description = readme_file.read().strip()

setup(
    name="aep-site-generator",
    version=VERSION,
    license="Apache 2.0",
    author="Luke Sneeringer",
    author_email="lukesneeringer@google.com",
    url="https://github.com/aep-dev/site-generator.git",
    packages=find_packages(exclude=["tests"]),
    description="Static site generator for aep.dev and forks.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points="""[console_scripts]
        aep-site-gen=aep_site.cli:publish
        aep-site-serve=aep_site.cli:serve
    """,
    platforms="Posix; MacOS X",
    include_package_data=True,
    install_requires=(
        "click==8.1.3",
        "flask==2.2.2",
        "itsdangerous==2.1.2",
        "jinja2==3.1.2",
        "markdown==3.4.1",
        "markupsafe==2.1.1",
        "pygments==2.13.0",
        "pymdown-extensions==9.7",
        "pyscss==1.4.0",
        "pyyaml==6.0.1",
        "six==1.16.0",
        "types-Markdown==3.4.2.1",
        "types-PyYAML==6.0.12",
        "werkzeug==2.2.2",
    ),
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Code Generators",
    ],
    zip_safe=False,
)
